// SPDX-License-Identifier: GPL-2.0
/*
* Copyright (C) 2020 Red Hat. All rights reserved.
*
* This file is released under the GPL.
*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/mman.h>
#include <unistd.h>

int main(int argc, char *argv[])
{
	int h;
	off_t size;
	unsigned char *map;
	unsigned seed;
	unsigned n_fuzz, i;

	if (argc < 2) fprintf(stderr, "argc!\n"), exit(1);
	h = open(argv[1], O_RDWR);
	if (h == -1) perror(argv[1]), exit(1);

	size = lseek(h, 0, SEEK_END);
	if (size == -1) perror("lseek"), exit(1);

	map = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, h, 0);
	if (map == MAP_FAILED) perror("mmap"), exit(1);

	if (argc >= 3) {
		seed = atoi(argv[2]);
	} else {
		seed = time(NULL);
	}
	fprintf(stderr, "fuzz seed %u\n", seed);
	srandom(seed);

	n_fuzz = random() % 256;

	for (i = 0; i < n_fuzz; i++) {
		unsigned r = random();
		unsigned r_hi = random();
		unsigned bit = r % 8;
		unsigned byte = (r_hi * ((unsigned long long)RAND_MAX + 1) + r) / 8 % size;
		map[byte] ^= 1 << bit;
	}

	if (munmap(map, size)) perror("munmap"), exit(1);

	if (close(h)) perror("close"), exit(1);

	return 0;
}
